#!/usr/bin/python
# -*- coding: utf-8 -*-
# Code Python 2.7

#   
#   _===_ _           _                  ______ _     _             _ _ _                 
#  |  ___| |         | |                 | ___ (_)   | |           (_) | |                
#  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __ 
#  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
#  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |   
#  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|   
#                                                                                         
# 
#  Fichier:   Interface_Charge_Fictive_Simple.py
#  Author: Électro-Bidouilleur   www.youtube.com
#  Mis a jour le 01.09.2015

# Importation de librairies appropriées
import time                # Permet l'accès à des fonctions de délai et d'heure
import thread              # Permet de rouler du code en multi-tâches
import sys                 # Permet entre autres de tuer le programme
import serial              # Pyserial. Doit être installé au préalable. Voir http://pyserial.sourceforge.net/

# VARIABLES GLOBALES
Vbatt = 0.00                           # La tension de la batterie
Ibatt = 0                              # Le courant de la batterie
Ibatt_Cible = 0                        # Le paramètre de courant cible à envoyer au PIC
Courant_Cible = 0                      # Le courant cible, exprimé en mA
Capacite_Batt = 0                      # LA capacité calculée de la batterie en mAh
Ser_Char = ''                          # Caractère série reçu.
Empecher_Lecture_Periodique = False    # Sémaphore utilisé pour contrôler l'accès au port série
heure_depart = 0.0                     # L'heure de départ du cycle de décharge, exprimé en secondes
Cycle_Decharge_Actif = False           # Indicateur d'Activité du cycle de décharge

         
# Tâche (thread) exécutée en parallàle à la boucle de saisie de commande.
# Gère la lecture de l'état de la charge fictive et dérive les tensions et courants du moment.           
def Lecture_Etat_Periodique():
   # Accès aux variables globales
   global Vbatt
   global Ibatt
   global Ibatt_Cible
   global Courant_Cible
   global Capacite_Batt
   global Ser_Char
   global Empecher_Lecture_Periodique
   global heure_depart
   global Cycle_Decharge_Actif
   while(1):   # Boucle perpétuelle
      if (Empecher_Lecture_Periodique == False):      # Vérifier que le sémaphore d'utilisation du port série est bas
         ser.write("E0000\n")       # Envoyer la commande de demande d'État du PIC Send the command to display 'État, including LF
         Ser_Char = ' '             # Initialiser le caractère Rx série avec un espace
         result = ''                # Initialiser la chaîne de caractères Rx série à vide
         while (Ser_Char <> "\x00"):  # Lire des caractères du port série jusqu'à la réception du caractère NULL (ASCII 0)
            try:  # tentative 
               Ser_Char = ser.read(1)  # Attendre un caractère et le lire
               result = result + Ser_Char # Concaténation à la chaîne de caractères
            except:  # Gestion d'erreur
               print "ERREUR !!!  Perdu la communication avec le système."
         Liste_Etat = result.split(',')      # séparation des différents champs de la commande d'état
         Vbatt = float(Liste_Etat[4]) / 1023 * 5 * 4     # Calcul de la tension de battereie en Volts
         print("Tension courante de la batterie (V): " + str(round(Vbatt,2)))    # Affichage de la tension
         if (Cycle_Decharge_Actif):    # Cycle de décharge actif?
            #  Oui, Tester l'arrêt automatique...
            if (Liste_Etat[1] == '0'):       #Indicateur de cycle de décharge désactivé?
               Cycle_Decharge_Actif = False     # Oui, désactiver le cycle de décharge              
               print ("Cycle de décharge terminé")    # Imprimer un message 
            else:   # Sinon faire les calculs de courant et capacité de batterie
               Ibatt = float(Liste_Etat[3]) / 1023 * 5 * 4 / 2 *1000    # Calcul du courant en milliAmpères
               print("Courant de décharge mesuré (mA): " + str(round(Ibatt,0)))     # Affichage du courant en milliAmpères
               Capacite_Batt = Courant_Cible * (time.time() - heure_depart) / 3600     # Calcul de capacité de la batterie
         print("Capacité de batterie calculé (mAh): " + str(round(Capacite_Batt,0)))    # Affichage de la capacité de la batterie
         if (Liste_Etat[1] == '1'): print ("Cycle de décharge en cours...")     # Affichage du statut du cycle de décharge
         else:  print ("Cycle de décharge inactif...")                          #   idem
         print("Entrez une commande (C, A, G, S, Q) : ")    # Affichage de l'invitation à une commande
      time.sleep(2)     # Paude de 2 secondes.
   
   
# CODE PRINCIPAL   

# Tentative d'ouverture du port série
try:  # Tentative
   print "En attente de l'ouverture du port série..."   # Message affiché
   ser = serial.Serial("/dev/ttyUSB1", 2400, timeout=1, xonxoff=False)     # Configurer le port série
   ser.flushInput() # Vider la mémoire tampon de réception du port série
except:  # Problème d'ouverture du port série
   print "ERREUR !!!  Impossible d'ouvrir le port série. Au revoir."     # Message d'erreur
   time.sleep(1)  # Pause 
   sys.exit()     # Quitter le programme

thread.start_new_thread(Lecture_Etat_Periodique, ())  # Lancement de la tâche parallèle permettant la lecture périodique de l'état du PIC.

# Boucle de traitement des commandes
while(1):
   # Traiter une nouvelle commande
   commande = raw_input("Entrez une commande (C, A, G, S, Q) : \n")     # Capture de la commande 
   Empecher_Lecture_Periodique = True     # Sémaphore pour empêcher l'accès simultané du port série
   # Traiter l'arrêt du cycle de décharge
   if (commande == 'S'):      # Commande d'arrêt de la décharge?
      Cycle_Decharge_Actif = False     # Oui, désactiver le cycle de décharge
      ser.write("S0000\n")             # Commande pour stopper le cycle de décharge
   # Traiter le changement de courant de décharge
   elif ((commande == 'C') and (Cycle_Decharge_Actif == False)):     # Commande de changement de courant? Cycle de décharge inactif?
      commande = raw_input("Entrez le Courant de Décharge (mA) : \n")      # Oui, capture du nouveau courant
      Courant_Cible = float(commande)     # Conversion numérique du courant
      Ibatt_Cible = round((float(commande) / 1000 * 1023 * 2 / 5 / 4), 0)        # Calcul du paramètre à envoyer au PIC
      ser.write("C" + str(Ibatt_Cible)[:-2].zfill(4) + "\n")         # Commande pour configurer courant de décharge
   # Traiter le changement de tension d'arrêt
   elif ((commande == 'A') and (Cycle_Decharge_Actif == False)):     # Commande de changement de tension d'arrêt? Cycle de décharge inactif?
      commande = raw_input("Entrez la Tension d'Arrêt Automatique de Décharge (V) : \n")      # Oui, capture de la nouvelle tension
      Tension_Arret = round((float(commande) / 4 * 1023 / 5), 0)       # Calcul du paramètre à envoyer au PIC
      ser.write("A" + str(Tension_Arret)[:-2].zfill(4) + "\n")       # Commande pour configurer la tension d'arrêt
   # Traiter le démarrage du cycle de décharge
   elif ((commande == 'G') and (Cycle_Decharge_Actif == False)):     # Commande de de démarrage du cycle de décharge? Cycle de décharge inactif?
      Cycle_Decharge_Actif = True      # Activer le cycle de décharge
      ser.write("G0000\n")       # Commande pour démarrer le cycle de décharge
      heure_depart = time.time()    # Capture de l'heure courant comme heure de départ, exprimée en secondes
      Capacite_Batt = 0       # Initialisation de la capacité de la batterie
   # Traiter la sortie du programme
   elif (commande == 'Q'):       # Commande de sortie de programme?
      ser.write("S0000\n")       # Commande pour stopper le cycle de décharge
      sys.exit()                 # Quitter le programme
   # Traiter les mauvaises commandes
   else: print("Erreur de commande!\n")     # Sinon, afficher le message d'erreur 
   Empecher_Lecture_Periodique = False      # Abaisser le sémaphore du port série
   time.sleep(0.1)      # Petite pause
   
   

